import React from 'react'
import { get, tail } from 'lodash'
import { reduxForm } from 'redux-form'
import styled from 'styled-components'
import { th } from '@pubsweet/ui-toolkit'
import { required } from 'xpub-validators'
import { compose, withProps } from 'recompose'
import { Button, Menu, ValidatedField } from '@pubsweet/ui'
import { withModal } from 'pubsweet-component-modal/src/components'

import {
  Row,
  Text,
  Label,
  Textarea,
  MultiAction,
  ContextualBox,
  ItemOverrideAlert,
  withFetching,
} from 'pubsweet-component-faraday-ui/src'

const options = [
  { value: 'publish', label: 'Publish' },
  { value: 'reject', label: 'Reject' },
  { value: 'minor', label: 'Request Minor Revision' },
  { value: 'major', label: 'Request Major Revision' },
]

const parseFormValues = ({ recommendation, ...rest }) => {
  const comments = Object.entries(rest).map(([key, value]) => ({
    content: value,
    public: key === 'public',
    files: [],
  }))

  return {
    comments,
    recommendation,
    recommendationType: 'editorRecommendation',
  }
}

const HERecommendation = ({
  formValues,
  handleSubmit,
  hasReviewerReports,
  highlight,
}) => (
  <ContextualBox
    highlight={highlight}
    label="Your Editorial Recommendation"
    mb={2}
  >
    <Root>
      <Row justify="flex-start">
        <ItemOverrideAlert flex={0} vertical>
          <Label required>Recommendation</Label>
          <ValidatedField
            component={input => (
              <Menu
                options={hasReviewerReports ? options : tail(options)}
                {...input}
              />
            )}
            name="recommendation"
            validate={[required]}
          />
        </ItemOverrideAlert>
      </Row>

      {get(formValues, 'recommendation') === 'publish' ||
      get(formValues, 'recommendation') === 'reject' ? (
        <Row mt={2}>
          <ItemOverrideAlert vertical>
            <Label required>Message for Author</Label>
            <ValidatedField
              component={Textarea}
              name="public"
              validate={[required]}
            />
          </ItemOverrideAlert>
        </Row>
      ) : (
        <ResponsiveRow mt={2}>
          <ResponsiveItem mr={1} vertical>
            <Label>
              Message for Author <Text secondary>Optional</Text>
            </Label>
            <ValidatedField component={Textarea} name="public" />
          </ResponsiveItem>

          <ResponsiveItem ml={1} vertical>
            <Label>
              Message for Editor in Chief <Text secondary>Optional</Text>
            </Label>
            <ValidatedField component={Textarea} name="private" />
          </ResponsiveItem>
        </ResponsiveRow>
      )}

      <Row justify="flex-end" mt={2}>
        <Button onClick={handleSubmit} primary size="medium">
          Submit recommendation
        </Button>
      </Row>
    </Root>
  </ContextualBox>
)

export default compose(
  withFetching,
  withModal(({ isFetching }) => ({
    isFetching,
    modalComponent: MultiAction,
  })),
  withProps(({ formValues }) => ({
    modalTitle: options.find(
      o => o.value === get(formValues, 'recommendation', 'publish'),
    ).label,
  })),
  reduxForm({
    form: 'HERecommendation',
    onSubmit: (
      values,
      dispatch,
      { onRecommendationSubmit, showModal, setFetching, modalTitle },
    ) => {
      showModal({
        title: `${modalTitle}?`,
        onConfirm: props => {
          onRecommendationSubmit(parseFormValues(values), {
            ...props,
            setFetching,
          })
        },
      })
    },
  }),
)(HERecommendation)

// #region styles
const Root = styled.div`
  display: flex;
  flex-direction: column;
  padding: ${th('gridUnit')};
`

const ResponsiveRow = styled(Row)`
  @media (max-width: 800px) {
    flex-direction: column;
  }
`

const ResponsiveItem = styled(ItemOverrideAlert)`
  @media (max-width: 800px) {
    margin-right: 0;
    margin-left: 0;
    width: 100%;
  }
`
// #endregion
